/***************************************/
//      Screen-Drawing Functions       //
/***************************************/

#include "CompileOpts.h"      // Compiler Options
#include <tigcclib.h>         // Library Header Files
#include "Defines.h"          // Numerical Constants
#include "Typedefs.h"         // Custom Defined Types
#include "Prototypes.h"       // Function Prototypes
#include "extgraph.h"         // Graphics Routines
#include "Globals.h"          // Global Variables



void draw_level_text (void)
{
memset(screenholder , 0 , LCD_SIZE);
//draw lvl text
Sprite16_OR(145 , 0 ,/*HEIGHT*/ 30 , (unsigned short int *)(((unsigned char*)lvls_ptr)+LVLPIC_OFFSET) , screenholder);
//draw level numbers
Sprite8_OR(145 , 31 ,/*HEIGHT*/ 20 , (((unsigned char *)lvls_ptr)+NUMPICS_OFFSET+20*(((level+1)/10)%10)) , screenholder);
Sprite8_OR(153 , 31 ,/*HEIGHT*/ 20 , (((unsigned char *)lvls_ptr)+NUMPICS_OFFSET+20*((level+1)%10)) , screenholder);
}



void draw_level (void)
{
signed short int xloop = -1,
                 yloop = -1,
                 temp;

while (++yloop < 25)
	{
	while (++xloop < 29)
		{
		temp = (signed short int) level_data[(yloop * 29) + xloop];
		if (temp)
			Sprite8_BLIT(xloop*5 , yloop*4 , TILE_HEIGHT , lvls_ptr+temp*4-4 , 0x07 , screenholder);
		}
	xloop = -1;
	}
}



void draw_score (void)
{
const char scoretext[6] = "SCORE", timetext[5] = "TIME";
unsigned long int temp_score;

PortSet(screenholder , 239 , 127);

// erases the block where the score is written
for (temp_score = 52; temp_score <= 99; temp_score++)
	*(((unsigned short int *)(screenholder))+(temp_score*15)+9) &= 0xC000;

if (game_type == PLAY_CLASSIC)
	{
	temp_score = score;
	FontSetSys(F_6x8);
	DrawChar(153 , 92, temp_score % 10 + '0' , A_NORMAL);
	temp_score /= 10;
	DrawChar(153 , 84, temp_score % 10 + '0' , A_NORMAL);
	temp_score /= 10;
	DrawChar(153 , 76, temp_score % 10 + '0' , A_NORMAL);
	temp_score /= 10;
	DrawChar(153 , 68, temp_score % 10 + '0' , A_NORMAL);
	temp_score /= 10;
	DrawChar(153 , 60, temp_score % 10 + '0' , A_NORMAL);
	temp_score /= 10;
	DrawChar(153 , 52, temp_score % 10 + '0' , A_NORMAL);

	FontSetSys(F_4x6);
	DrawChar(147 , 61, scoretext[0] , A_NORMAL);
	DrawChar(147 , 67, scoretext[1] , A_NORMAL);
	DrawChar(147 , 73, scoretext[2] , A_NORMAL);
	DrawChar(147 , 79, scoretext[3] , A_NORMAL);
	DrawChar(147 , 85, scoretext[4] , A_NORMAL);
	}
else
	{
	temp_score = limiting_factor;
	FontSetSys(F_6x8);
	temp_score /= 60;
	DrawChar(153 , 68, temp_score % 10 + '0' , A_NORMAL);
	temp_score /= 10;
	DrawChar(153 , 60, temp_score % 10 + '0' , A_NORMAL);
	temp_score /= 10;
	DrawChar(153 , 52, temp_score % 10 + '0' , A_NORMAL);

	DrawChar(153 , 76, ':' , A_NORMAL);
	temp_score = (limiting_factor % 60);
	DrawChar(153 , 92, temp_score % 10 + '0' , A_NORMAL);
	temp_score /= 10;
	DrawChar(153 , 84, temp_score + '0' , A_NORMAL);

	FontSetSys(F_4x6);
	DrawChar(147 , 64, timetext[0] , A_NORMAL);
	DrawChar(148 , 70, timetext[1] , A_NORMAL);
	DrawChar(146 , 76, timetext[2] , A_NORMAL);
	DrawChar(147 , 82, timetext[3] , A_NORMAL);
	}

PortRestore();
}



void draw_pacman (void)
{
if (pacman.ani_delay<(WAIT_FRAMES/2)) // mouth closed
	Sprite8_OR(pacman.x*5 + pacman.xshift - 1, pacman.y*4 + pacman.yshift - 1 , OBJ_HEIGHT , ((unsigned char*)lvls_ptr)+OBJPICS_OFFSET+(pacman.direction*12) , LCD_MEM);
else // mouth open
	Sprite8_OR(pacman.x*5 + pacman.xshift - 1, pacman.y*4 + pacman.yshift - 1 , OBJ_HEIGHT , ((unsigned char*)lvls_ptr)+OBJPICS_OFFSET+6+(pacman.direction*12) , LCD_MEM);
}



void draw_ghosts (void)
{
signed short int loop = -1, draw_vulnerable = 0, temp = 0;

while (++loop < 4)
	{
	temp = ghost[loop].vulnerable;

	if (temp > 150) // vulnerable ghost
		draw_vulnerable = 1;
	else if (!temp) // normal ghost
		draw_vulnerable = 0;
	else // blinking ghost
		{
		if (temp % 20 < 10)
			draw_vulnerable = 0;
		else
			draw_vulnerable = 1;
		}


	if (draw_vulnerable) // vulnerable ghost
		Sprite8_BLIT(ghost[loop].x*5 + ghost[loop].xshift - 1, ghost[loop].y*4 + ghost[loop].yshift - 1 , OBJ_HEIGHT , ((unsigned char*)lvls_ptr)+OBJPICS_OFFSET+54 , 0x01 , LCD_MEM);
	else // normal ghost
		Sprite8_BLIT(ghost[loop].x*5 + ghost[loop].xshift - 1, ghost[loop].y*4 + ghost[loop].yshift - 1 , OBJ_HEIGHT , ((unsigned char*)lvls_ptr)+OBJPICS_OFFSET+48 , 0x01 , LCD_MEM);
	}
}



void draw_fruit (void)
{
if (fruit.is_on_screen)
	Sprite8_BLIT(fruit.x*5+fruit.xshift-1,fruit.y*4+fruit.yshift-1,OBJ_HEIGHT,((unsigned char*)lvls_ptr)+OBJPICS_OFFSET+60+6*(fruit.f_type) , 0x01 , LCD_MEM);
}