/***************************************/
//         Game Play Functions         //
/***************************************/

#include "CompileOpts.h"      // Compiler Options
#include <tigcclib.h>         // Library Header Files
#include "Defines.h"          // Numerical Constants
#include "Typedefs.h"         // Custom Defined Types
#include "Prototypes.h"       // Function Prototypes
#include "extgraph.h"         // Graphics Routines
#include "Globals.h"          // Global Variables



void play_pacman (void)
{
init_globals();

while (level + 1)
	{
	get_level_data();

	play_level();
	}

game_over();
high_score();
}



void get_level_data (void) //initializes level_data, limiting_factor, and hot_spots
{
signed short int loop = -1, possible_dirs = 0, counter = 0;
unsigned char * data_file = (lvls_ptr)+LEVELS_OFFSET+(((level%NUM_OF_LEVELS) * SIZE_OF_LEVEL));

memcpy(level_data , data_file , SIZE_OF_LEVEL);

while (++loop < SIZE_OF_LEVEL) //loop through all spots looking for dots and count them
	{
	if ((level_data[loop] == 1)||(level_data[loop] == 2))
		{
		counter++;
		if (game_type == PLAY_TIMED) //timed mode has no dots, so eliminate them
			{
			level_data[loop] = 0;
			}
		}
	}

if (game_type == PLAY_CLASSIC) limiting_factor = counter;
else limiting_factor = level*10+20;

loop = -1;
while (loop++ < SIZE_OF_LEVEL)
	{
	possible_dirs = 0;
	if (level_data[loop+1] < 3) // space to the right
		possible_dirs++;
	if (level_data[loop-1] < 3) // space to the left
		possible_dirs++;
	if (level_data[loop-29] < 3) // space above
		possible_dirs++;
	if (level_data[loop+29] < 3) // space below
		possible_dirs++;
	
	if (possible_dirs > 2)
		hot_spots[loop] = 1;
	else
		hot_spots[loop] = 0;
	}
hot_spots[9*29+14] = 1;
}



void spawn_pacman (void)
{
pacman.x = 14;
pacman.y = 15;
pacman.xshift = 0;
pacman.yshift = 0;
pacman.direction = RIGHT;
pacman.ani_delay = 0;
pacman.ghosts_eaten = 0;
pacman.last_dir_pressed = RIGHT;
}



void spawn_ghost (signed short int ghost_num)
{
signed short int start_pos[4][2] = {{14,11},{12,12},{16,12},{14,13}};

ghost[ghost_num].x = start_pos[ghost_num][0];
ghost[ghost_num].y = start_pos[ghost_num][1];
ghost[ghost_num].xshift = 0;
ghost[ghost_num].yshift = 0;
ghost[ghost_num].direction = RIGHT;
ghost[ghost_num].vulnerable = 0;
ghost[ghost_num].cage_delay = 120;
ghost[ghost_num].last_dir_pressed = RIGHT;
}



void spawn_fruit (void)
{
fruit.x = 28 * random(2);
fruit.y = 12;
fruit.xshift = 0;
fruit.yshift = 0;
fruit.direction = (fruit.x / 14); //if x=0, direction=0=RIGHT... otherwise x=28 and direction=2=LEFT
if (!random(7)) fruit.f_type = HEART;
else fruit.f_type = (random(2)+1);
fruit.is_on_screen = TRUE;
fruit.last_dir_pressed = fruit.direction;
}



void play_level (void)
{
unsigned short int counter, loop, time_counter;
while (lives >= 0)
	{
	spawn_pacman();
	spawn_ghost(0);
	spawn_ghost(1);
	spawn_ghost(2);
	spawn_ghost(3);
	fruit.is_on_screen = FALSE;
	counter = 0;
	time_counter = 0;
	ghost[0].cage_delay = 60;
	ghost[1].cage_delay = 120; // these are changed so
	ghost[2].cage_delay = 180; // the ghosts come out
	ghost[3].cage_delay = 240; // one by one first
	draw_level_text();
	draw_level();
	draw_score();

	FastCopyScreen(screenholder , LCD_MEM);
	//memcpy(LCD_MEM , screenholder , 100*30-10);

	draw_pacman();
	draw_ghosts();

	if (text_box("Ready...",MSG_INFO)==FALSE)
		{
		level = -1;
		return;
		}

	wait(MENU_WAIT);

//this while loop evaluates true if there are dots in the level or
//time on the clock, depending on which game type you are playing
	while (limiting_factor>0)
		{
		move_obj(&pacman);//move pacman every frame

		for (loop=0; loop<4; loop++)
			{
			if (ghost[loop].vulnerable)//if vulnerable, ghosts move 2 of 3 frames
				{
				if (counter)
					move_obj(&ghost[loop]);
				}
			else //otherwise they move every frame
				{
				move_obj(&ghost[loop]);
				}
			}

		if (++counter==3) //fruit moves only 2 of every 3 frames
			counter=0;
		else
			move_obj(&fruit);


		if (game_type==PLAY_TIMED) // increment counter for timed mode
			{
			if (++time_counter==28) //approx 1 second
				{
				limiting_factor--;
				score++;
				draw_score();
				time_counter=0;
				}
			}

		FastCopyScreen(screenholder , LCD_MEM);
		//memcpy(LCD_MEM , screenholder , 100*30-10);
		draw_fruit();
		draw_ghosts();
		draw_pacman();

		if (level_data[pacman.y*29 + pacman.x]==1) //if on small dot
			eat_dot();

		if (level_data[pacman.y*29 + pacman.x]==2) //if on big dot
			{
			start_ghost_count();
			eat_dot();
			}

		if (fruit_collide()==TRUE) // check if on fruit
			{
			kill_fruit();
			}
		if (ghost_collide()==TRUE) //check if on ghost
			{
			lives--;
			break;
			}

		if (!(random(30*5) || fruit.is_on_screen))//randomly spawn fruit
			spawn_fruit();

		if (_keytest(RR_ESC))
			{
			level = -1;
			lives = -1;
			break;
			}
		if (_keytest(RR_2ND) || _keytest(RR_ENTER))
			{
			pause();
			continue;
			}
		wait(WAIT_QUANTA);
		}

	if (limiting_factor <= 0) // if you beat the level
		{
		level++;
		return;
		}
	}
level = -1;
}



void eat_dot (void)
{
if (game_type==PLAY_CLASSIC) score += 10;
draw_score();

DrawGrayRect2B(pacman.x*5,pacman.y*4,pacman.x*5+4,pacman.y*4+3,COLOR_WHITE,RECT_FILLED,screenholder,screenholder);

level_data[pacman.y*29 + pacman.x]=0;
if (game_type==PLAY_CLASSIC) limiting_factor--;
}



void start_ghost_count (void)
{
signed short int temp = level * 15;
if (temp>300) temp=300;
if (game_type==PLAY_CLASSIC) score += 10;
ghost[0].vulnerable = 300 - temp;
ghost[1].vulnerable = 300 - temp;
ghost[2].vulnerable = 300 - temp;
ghost[3].vulnerable = 300 - temp;
pacman.ghosts_eaten = 0;
}



inline void kill_fruit (void)
{
fruit.is_on_screen = FALSE;
}



inline void game_over (void)
{
text_box("GAME OVER",MSG_GO);
}





