/***************************************/
//          Utility Functions          //
/***************************************/

#include "CompileOpts.h"      // Compiler Options
#include <tigcclib.h>         // Library Header Files
#include "Defines.h"          // Numerical Constants
#include "Typedefs.h"         // Custom Defined Types
#include "Prototypes.h"       // Function Prototypes
#include "extgraph.h"         // Graphics Routines
#include "Globals.h"          // Global Variables



void blur_screen (void)
{
signed short int count = LCD_SIZE;
while (count--)
	*(((unsigned char*)0x4C00)+count) &= (((count%60) < 30)? 0x55:0xAA);
}



BOOL text_box (const char *buffer,unsigned short int type)
{
FontSetSys(F_6x8);
blur_screen();
//draw rectangles
DrawGrayRect2B(24,20,122,76,COLOR_BLACK,RECT_FILLED,LCD_MEM,LCD_MEM);
DrawGrayRect2B(25,35,120,74,COLOR_WHITE,RECT_FILLED,LCD_MEM,LCD_MEM);
//draw title bar
DrawStr(40,25,PACMAN_VERSION_STRING,A_REVERSE);
//draw string
DrawStr(73-((DrawStrWidth(buffer,F_6x8))>>1),47-7*type,buffer,A_NORMAL);
//draw lives info
if (type != MSG_GO)
	{
	Sprite8_OR(58,56,OBJ_HEIGHT,lvls_ptr+68,LCD_MEM);
	DrawChar(70,55,'x',A_NORMAL);
	DrawChar(82,55,(lives%10)+'0',A_NORMAL);
	}
//draw score info
DrawStr(37,64,"Score ",A_NORMAL);
if (game_type==PLAY_CLASSIC)
	{
	DrawChar(73,64,((score/100000)%10)+'0',A_NORMAL);
	DrawChar(79,64,((score/10000)%10)+'0',A_NORMAL);
	DrawChar(85,64,((score/1000)%10)+'0',A_NORMAL);
	DrawChar(91,64,((score/100)%10)+'0',A_NORMAL);
	DrawChar(97,64,((score/10)%10)+'0',A_NORMAL);
	//DrawChar(103,64,(score%10)+'0',A_NORMAL);
	}
else
	{
	DrawChar(73,64,((score/6000)%10)+'0',A_NORMAL);
	DrawChar(79,64,((score/600)%10)+'0',A_NORMAL);
	DrawChar(85,64,((score/60)%10)+'0',A_NORMAL);
	DrawChar(91,64,':',A_NORMAL);
	DrawChar(97,64,((score%60)/10)+'0',A_NORMAL);
	//DrawChar(103,64,(score%10)+'0',A_NORMAL);
	}
DrawChar(103,64,(score%10)+'0',A_NORMAL);

while (!(_keytest(RR_ENTER) || _keytest(RR_2ND) || _keytest(RR_ESC)))
	idle();

if (_keytest(RR_ESC))
	return FALSE;

return TRUE;
}



//=======================================================
//   VTI detection routine (courtesy of Greg Dietsche)
//=======================================================
extern BOOL is_vti (void);
asm("
is_vti:
	trap #12
	move.w %d0,%d1
	move.w #0x6000,%sr
	move %sr,%d0
	lsr.w #8,%d0
	lsr.b #6,%d0
	move.w %d1,%sr
	rts
");



unsigned char * get_data_ptr (unsigned short int flag)
{
SYM_ENTRY *symentry;
HANDLE pmhandle;
unsigned char *data_file;

if (flag==DATA_FILE) symentry = SymFindPtr(SYMSTR("pacmlvls"),0);
else symentry = SymFindPtr(SYMSTR("pacmhscr"),0);	

if (symentry == NULL)
	return NULL;

if ((pmhandle = HeapMoveHigh(symentry->handle)) == H_NULL)
	pmhandle = symentry->handle;
data_file = (unsigned char*)HLock(pmhandle);
return (data_file+2);
}



BOOL initialize (void)
{
unsigned char * data_ptr;

level_data = NULL;
hot_spots = NULL;
lvls_ptr = NULL;
hscr_ptr = NULL;
screenholder = NULL;

save_ai1 = GetIntVec(AUTO_INT_1);
SetIntVec(AUTO_INT_1 , DUMMY_HANDLER);
save_ai5 = GetIntVec(AUTO_INT_5);
SetIntVec(AUTO_INT_5 , DUMMY_HANDLER);

EM_moveSymFromExtMem(SYMSTR("pacmlvls"),HS_NULL); //unarchive the data file
EM_moveSymFromExtMem(SYMSTR("pacmhscr"),HS_NULL); //unarchive the hscr file

data_ptr = get_data_ptr(DATA_FILE);
hscr_ptr = get_data_ptr(HSC_FILE);

if ((data_ptr==NULL)||(hscr_ptr==NULL))
	return FALSE;

if (!ttunpack_valid(data_ptr))
	return FALSE;

if ((lvls_ptr = malloc(ttunpack_size(data_ptr)))==NULL)
	return FALSE;

ttunpack_decompress(data_ptr,lvls_ptr);

randomize();
init_globals();

if ((level_data = malloc(SIZE_OF_LEVEL))==NULL)
	return FALSE;
if ((screenholder = malloc(LCD_SIZE))==NULL)
	return FALSE;
if ((hot_spots = malloc(SIZE_OF_LEVEL))==NULL)
	return FALSE;

is_on_vti = is_vti();

return TRUE;
}



void init_globals (void)
{
level = 0;
score = 0;
lives = 4;
limiting_factor=10;
}



void quit (void)
{
HeapUnlock((SymFindPtr(SYMSTR("pacmlvls"),0))->handle);
HeapUnlock((SymFindPtr(SYMSTR("pacmhscr"),0))->handle);

if (level_data != NULL) free(level_data);
if (screenholder != NULL) free(screenholder);
if (hot_spots != NULL) free(hot_spots);
if (lvls_ptr != NULL) free(lvls_ptr);

SetIntVec(AUTO_INT_1,save_ai1);
SetIntVec(AUTO_INT_5,save_ai5);

EM_moveSymToExtMem(SYMSTR("pacmlvls"),HS_NULL);//archive the data file
EM_moveSymToExtMem(SYMSTR("pacmhscr"),HS_NULL);//archive the hscr file
}



void pause (void)
{
PortRestore();
DrawStrExt(0,45,"PAUSED",A_NORMAL|A_CENTERED,F_8x10);
wait(MENU_WAIT);
while (!(_keytest(RR_ENTER) || _keytest(RR_2ND) || _keytest(RR_ESC)))
	idle();
FastCopyScreen(screenholder,LCD_MEM);
draw_pacman();
draw_ghosts();
draw_fruit();
wait(MENU_WAIT);
}



void wait (unsigned long int length)
{
unsigned long int loop;
volatile unsigned short int counter;

if (!is_on_vti)
	length *= 2;

for (loop=0; loop<length; loop++) counter++;
}



void DrawStrExt (signed short int x , signed short int y , const char *buffer , signed short int attrib , signed short int font)
{
FontSetSys(font);

if (attrib&A_CENTERED)
	x=(72-(DrawStrWidth(buffer,font)>>1));

DrawStr(x,y,buffer,(attrib&A_CENTERED)?(attrib-A_CENTERED):(attrib));
}